#include "cquad.h"

double cquad_dqawo(double f(), double a, double b, double omega, int integr, double epsabs, double epsrel, double *abserr, int *neval, int *ier, int *last, int *iwork, double *work)
{
/******************************************************************************
***  Date written:         03/31/2007  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Last revision date:   11/02/2012  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Author:    Gianluca Russo    (gianlurusso@fastwebnet.it)               ***
***             Salvatore Cuomo   (salvatore.cuomo@unina.it)                ***
***                                                                         ***
***             University of Naples "Federico II"                          ***
***             Dipartimento di Matematica e Applicazioni "R. Caccioppoli"  ***
***									    ***
*** 									    ***
***									    ***
*** This code is provided "as is", without any warranty of any kind,        ***
*** either expressed or implied, including but not limited to, any implied  ***
*** warranty of merchantibility or fitness for any purpose. In no event     ***
*** will any party who distributed the code be liable for damages or for    ***
*** any claim(s) by any other party, including but not limited to, any      ***
*** lost profits, lost monies, lost data or data rendered inaccurate,	    ***
*** losses sustained by third parties, or any other special, incidental or  ***	
*** consequential damages arising out of the use or inability to use the    ***
*** program, even if the possibility of such damages has been advised	    ***	
*** against. The entire risk as to the quality, the performance, and the    ***	
*** fitness of the program for any particular purpose lies with the party   ***
*** using the code.							    ***
***									    ***
*** This code, and any derivative of this code, may not be used in a        ***
*** commercial package without the prior explicit written permission of     ***
*** the authors. Verbatim copies of this code may be made and distributed   ***
*** in any medium, provided that this copyright notice is not removed or    ***
*** altered in any way. No fees may be charged for distribution of the      ***
*** codes, other than a fee to cover the cost of the media and a            ***
*** reasonable handling fee.						    ***
***-------------------------------------------------------------------------***
***  Purpose:                                                               ***
***                                                                         ***
***    The function provides a C interface to the fortran routine dqawo     ***
***    of quadpack package.                                                 ***
***                                                                         ***
***    The function calculates an approximation result to a given definite  ***
***    integral i = integral of f(x) * w(x) over (a, b), where:             ***
***                                                                         ***
***                    w(x) = cos(omega * x)   or                           ***
***                    w(x) = sin(omega * x)                                ***
***                                                                         ***
***    hopefully satisfying following claim for accuracy:                   ***
***                                                                         ***
***              abs(i - result) <= max(epsabs, epsrel * abs(i))            ***
***-------------------------------------------------------------------------***
***  Description:                                                           ***
***                                                                         ***
***    A routine for the integration of COS(omega * x) * f(X) or            ***
***    SIN(omega * x) * f(X) over a finite interval (a,b).                  ***
***    omega is specified by the user.                                      ***
***    The rule evaluation component is based on the modified               ***
***    Clenshaw-Curtis technique. An adaptive subdivision scheme is used    ***
***    connected with an extrapolation procedure, which is a modification   ***
***    of that in cquad_dqags and provides the possibility to deal even     ***
***    with singularities in f.                                             ***
***                                                                         ***
***    Double precision version.                                            ***
***-------------------------------------------------------------------------***
***  Input:                                                                 ***
***                                                                         ***
***    f        function subprogram defining the integrand function f(x)    ***
***                                                                         ***
***    a        lower limit of integration                                  ***
***                                                                         ***
***    b        upper limit of integration                                  ***
***                                                                         ***
***    omega    parameter in the integrand weight function                  ***
***                                                                         ***
***    integr   indicates which of the weight functions is used:            ***
***                                                                         ***
***                 integr = 1 -->     w(x) = cos(omega * x)                ***
***                 integr = 2 -->     w(x) = sin(omega * x)                ***
***                                                                         ***
***    epsabs   absolute accoracy requested                                 ***
***                                                                         ***
***    epsrel   relative accoracy requested                                 ***
***-------------------------------------------------------------------------***
***  Output:                                                                ***
***                                                                         ***
***    res      approximation to the integral                               ***
***                                                                         ***
***    abserr   estimate of the modulus of the absolute error               ***
***                                                                         ***
***    neval    number of integrand evaluations                             ***
***                                                                         ***
***    ier      normal termination flag:                                    ***
***                                                                         ***
***               ier = 0 --> normal and reliable termination of the        ***
***                           function. It is assumed that the requested    ***
***                           accuracy has been achieved.                   ***
***                                                                         ***
***               ier = 1 --> maximum number of subdivisions allowed        ***
***                           (= leniw / 2) has been achieved.              ***
***                                                                         ***
***               ier = 2 --> the occurrence of roundoff error is detected  ***
***                           which prevents the requested tolerance from   ***
***                           being achieved.                               ***
***                           The error may be under-estimated.             ***
***                                                                         ***
***               ier = 3 --> extremely bad integrand behaviour occurs at   ***
***                           some points of the integration interval.      ***
***                                                                         ***
***               ier = 4 --> the algorithm does not converge. Roundoff     ***
***                           error is detected in the extrapolation table. ***
***                           It is presumed that the requested tolerance   ***
***                           cannot be achieved due to roundoff in the     ***
***                           extrapolation table, and that the returned    ***
***                           result is the best which can be obtained.     ***
***                                                                         ***
***               ier = 5 --> the integral is probably divergent, or slowly ***
***                           convergent. it must be noted that divergence  ***
***                           can occur with any other value of ier > 0.    ***
***                                                                         ***
***               ier = 6 --> the input is invalid because (epsabs <= 0 and ***
***                           epsrel < max(50 * rel.mach.acc., 0.5d-28)) or ***
***                           (integr != 1 and integr != 2), or leniw < 2   ***
***                           or maxp1 < 1 or lenw < (leniw * 2 + maxp1     ***
***                           * 25).                                        ***
***                           In all these events res, abserr, neval and    ***
***                           last are set to zero.                         ***
***                                                                         ***
***    last     number of subintervals produced in the subdivision process  ***
***-------------------------------------------------------------------------***
***  Workspace:                                                             ***
***                                                                         ***
***    leniw    dimensioning parameter for iwork.                           ***
***             leniw / 2 equals the maximum number of subintervals allowed ***
***             in the partition of the given integration interval (a,b)    ***
***             (leniw > 1)                                                 ***
***                                                                         ***
***    maxp1    gives an upper bound on the number of chebyshev moments     ***
***             which can be stored, i.e. for the intervals of lengths      ***
***             abs(b - a) * 2 * *(-l), l = 0, 1, ..., maxp1 - 2            ***
***             (maxp1 > 0)                                                 ***
***                                                                         ***
***    lenw     dimensioning value for work.                                ***
***             (lenw must be at least leniw * 2 + maxp1 * 25)              ***
***                                                                         ***
***    iwork    vector of dimension at least leniw, the first k elements    ***
***             of which contain pointers to the error estimates over the   ***
***             subintervals, such that work(limit * 3 + iwork(1)), ...,    ***
***             work(limit * 3 + iwork(k)) form a decreasing sequence, with ***
***             limit = lenw/2 , and k = last if last <= (limit/2 + 2), and ***
***             k = limit + 1 - last otherwise                              ***
***                                                                         *** 
***    work     vector of dimension at least lenw.                          ***
***             On return:                                                  ***
***                work(1), ..., work(last) contain the left end points of  ***
***                  the subintervals in the partition of (a,b)             ***
***                work(limit + 1), ..., work(limit + last) contain the     ***
***                  right end points                                       ***
***                work(limit * 2 + 1), ..., work(limit * 2 + last) contain ***
***                  the integral approximations over the subintervals      ***
***                work(limit * 3 + 1), ..., work(limit * 3 + last) contain ***
***                  the error estimates                                    ***
***                work(limit * 4 + 1), ..., work(limit * 4 + maxp1 * 25)   ***
***                  provide space for storing the chebyshev moments        ***
***             (note that limit = lenw / 2)                                ***
***-------------------------------------------------------------------------***
***  References:      QUADPACK - Piessens, de Doncker, Uberhuber, Kahaner   ***
***                                                                         ***
***                   http://netlib.org                                     ***
******************************************************************************/

    int leniw = LENIW, lenw = LENW, maxp1 = MAXP1;
    double res;

    dqawo_(f, &a, &b, &omega, &integr, &epsabs, &epsrel, &res, abserr, neval, ier, &leniw, &maxp1, &lenw, last, iwork, work);

    return res;
}
